library(matlab)  # calling for "meshgrid"
##############################################################
## File: Bilinear.R
##
## This program is for the simple Bilinear model:
## Y_{t}= 0.9 Y_{t-1}+0.3 varepsilon_{t}Y_{t-1}+varepsilon_{t}
## Coded by: Dong Li 
##
## Reference:
## Li, D. and Qiu, J. (2020).
##  The marginal density of a TMA(1) process. 
##  Journal of Time Series Analysis, 41, 476--484.
##############################################################
##############################################################
##########  For Lyapunov exponent ############################
##############################################################
f1 = function(x){
f  = log(x)*exp(-x^2/2)*sqrt(2/pi)
return(f)
}
#######################################
betamin = 0
betamax = exp(-integrate(f1, lower=0, upper=Inf)$value)
#######################################
##############################################################
f2 = function(x,alpha,beta){
f  = log(abs(alpha^2-beta^2*x^2))*exp(-x^2/2)
return(f)
}
#######################################
##############################################################
ALPHA.root = function(beta){
alphalow   = -sqrt(3)
alphaupp   = sqrt(2)

while (alphaupp-alphalow>1e-8){
Ilow=integrate(f2, lower=0, upper=Inf,alpha=alphalow, beta=beta)$value
Imid=integrate(f2, lower=0, upper=Inf,alpha=(alphalow+alphaupp)/2, beta=beta)$value
Iupp=integrate(f2, lower=0, upper=Inf,alpha=alphaupp, beta=beta)$value

if (Imid>0){alphaupp=(alphalow+alphaupp)/2}
else{alphalow = (alphalow+alphaupp)/2}
}
return((alphalow+alphaupp)/2)
}
#######################################
beta      = seq(betamin,betamax,by=0.01)
ALPHABETA = sapply(beta,ALPHA.root)

plot(c(-betamax,-rev(beta),beta,betamax), c(0,rev(ALPHABETA), ALPHABETA,0),type="l",col="blue",las=1,lwd=2,
    xlab="",ylab="",cex.lab=1.5,cex.axis=1.5,ylim=c(-2.2,2.2))
    #main=expression(paste("(a)  ", epsilon[t], "~",italic(N),"(0,1)")),cex.main=3)
lines(c(-betamax,-rev(beta),beta,betamax), -c(0,rev(ALPHABETA), ALPHABETA,0),type="l",col="blue",las=1,lwd=2)
polygon(c(-betamax,-rev(beta),beta,betamax), c(0,rev(ALPHABETA), ALPHABETA,0), 
        density = NULL, angle = 45,border = "cyan", col = "grey",lwd=2)
polygon(c(-betamax,-rev(beta),beta,betamax), -c(0,rev(ALPHABETA), ALPHABETA,0), 
        density = NULL, angle = 45,border = "cyan", col = "grey",lwd=2)

abline(h=0,v=0,lwd=3)
arrows(0,0,0,2.4, length = 0.2, angle = 25,cex=2,lwd=3)
arrows(0,0,2,0, length = 0.2, angle = 25,cex=2,lwd=3)
text(1.7,-0.2,expression(beta),cex=3)
text(0.2,2.1,expression(alpha),cex=3)


##############################################################
##########  The function K(x, y)  ############################
##############################################################
K.fun=function(z,th){
#th=c(alpha,beta)
     f = pnorm((z$x-th[1]*z$y)/abs(1+th[2]*z$y))
return(f)
}
##############################################################
##############################################################
g.den = function(x,XX,G,th){
dif   = c(G,1)-c(0,G)
fden  = sum((dnorm((x-th[1]*XX)/abs(1+th[2]*XX))/abs(1+th[2]*XX))*dif)
return(fden)
}
##############################################################
th = c(0.9,0.3)
x  = seq(-4,6,by=0.01)
n  = length(x)
y  = (c(min(x)-1,x)+c(x, max(x)+1))/2
z  = meshgrid(x,y)

A    = t(K.fun(z,th))
K    = A[,1:n]-A[,2:(n+1)]
a    = A[,n+1]
BL.G = solve(eye(n)-K)%*%a
BL.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=BL.G,th)

############# Plot the density function ##########################
par(mfrow=c(1,2))
plot(x,BL.g.den,type="l",col="blue",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(a)  ", beta==0.3),adj=0),
  cex.main=2,ylim=c(0,0.4))
abline(h=0,lty=2)

##############################################################
##############################################################
##############################################################

##########################################################
th = c(0.9,1.5)        # AR and BL parameters
x  = seq(-4,6,by=0.01)
n  = length(x)
y  = (c(min(x)-1,x)+c(x, max(x)+1))/2
z  = meshgrid(x,y)

A    = t(K.fun(z,th))
K    = A[,1:n]-A[,2:(n+1)]
a    = A[,n+1]
BL.G = solve(eye(n)-K)%*%a
BL.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=BL.G,th)

############# Plot the density function ##########################
plot(x,BL.g.den,type="l",col="blue",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(b)  ", beta==1.5),adj=0),
  cex.main=2,ylim=c(0,2))
abline(h=0,lty=2)


############# Plot the density function ##########################
#par(mfrow=c(1,2))
#plot(x,DAR.g.den,type="l",col="red",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
#  xlab="x",ylab="",main=expression(paste("(b)  ", alpha==1),adj=0),
#  cex.main=2,ylim=c(0,0.5))
#abline(h=c(0,1),lty=2)


##################################################################