%%  File: pdf_asMA11_Normal.m  
%%
%%  Coded by: JDG
%%
%%  pdf of asMA(1,1) process with
%%  normal N(0,var_w) innovations w_{t}
%%  see: Wecker(JASA,1981) and Brannas & De Gooijer(JoF, 1994)
%%
%%    Y_{t} =  w_{t} + beta_p w_{t-1} I(w_{t-1}>= s)
%%                   + beta_m w_{t-1} I(w_{t-1}< s) 
%%
%% OUTPUT: Part 1
%% Parameters: beta_p, beta_m
%%    0.5000   -0.5000
%% Non-central moments asMA11: nu_1     nu_2       nu_3     nu_4
%%                            0.3989    1.2500    1.3963    4.6875
%% asMA11: var     skew      kurt     rho(ACF(1)) 
%%       1.0908    0.0239    3.0060         0
%%
%% Part 2:
%% Parameters for SETAR(2;1,1) = asMA(1,1): alpha, beta_p
%%    0.4472    0.5000
%% SETAR211: nu_1    nu_2      nu_3      nu_4
%%         0.3989    1.2500    1.3963    4.6875
%% SETAR211: 
%%    var       skew      kurt    rho at lag 1  rho2 
%%    1.0908    0.0239    3.0060    0.1359      0.1359
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

beta_p  = 0.5;
beta_m  = -0.5;
par     = [ beta_p, beta_m];

s       = 0;  % threshold parameter
var_w   = 1;
sd_w    = sqrt(var_w);
Umax    = 201;

pdf_asMA = zeros(1,Umax);
index    = zeros(1,Umax);

var_p = ( 1 + beta_p^(2) )*var_w;  % plus
var_m = ( 1 + beta_m^(2) )*var_w;  % min
sd_p  = sqrt(var_p);
sd_m  = sqrt(var_m);

delta = beta_p - beta_m;
eta   = beta_p + beta_m;
psi   = beta_p^(2) + beta_m^(2);
gamma = psi + beta_p*beta_m;

pdf_asMA  = zeros(1,201);
pdf_asMA2 = zeros(1,201);
index     = zeros(1,201);

for i=1:Umax
  u       = -10+(i-1)*0.1;
  T_p     = (-beta_p*u)/var_p + (s*var_p)/var_w;
  T_m     = ( beta_m*u)/var_m - (s*var_m)/var_w;  
  f_asMA  = normcdf(T_p) * normpdf(u,0,var_p) + normcdf(T_m) * normpdf(u,0,var_m); 

  if (beta_p == -beta_m)
%   sd_p = 1;
    f_asMA2 = ( 2/(sd_p*sqrt(2*pi) ) ) * exp( -u^(2)/(2*(1+beta_p^(2))) ) * normcdf(-beta_p*u/sd_p);
  end
  
  pdf_asMA(i)  = f_asMA;
  pdf_asMA2(i) = f_asMA2;
  index(i)     = u;
end

figure(1)
plot(index,pdf_asMA)

%%%%%%%%%%%%%%    PART 1      %%%%%%%%%%%%%%%%%%%%%%%
nu_1_asMA = (beta_p - beta_m)*sd_w/sqrt(2*pi); % mean, 1st central moment
nu_2_asMA = 0.5*(1+beta_p^(2))*var_w + 0.5*(1+beta_m^(2))*var_w; % 2nd central moment

var_asMA  = nu_2_asMA - nu_1_asMA*nu_1_asMA;  % ACVF(0) = variance of asMA(1,1) process
sd_asMA   = sqrt(var_asMA);

%%  Brannas & DeGooijer (JoF, 1994)
nu_3_asMA = ( 3*delta*sd_w^(3)/sqrt(8*pi) ) * (2 + (4/3)*gamma );  % 3rd central moment
nu_4_asMA = 3*sd_w^(4)*( 1 +0.5*(beta_p^(4) + beta_m^(4)) + psi ); % 4th  ,,     moment

%% Next a check: This should be the same as "nu_4_asMA" above in the special case
% nu_4 = 3*(1+beta_p^(4) +2*beta_p^(2)); % for beta_p = beta_m =beta, sd_w = 1

%% Central moments
mu3_asMA = nu_3_asMA - 3* nu_2_asMA * nu_1_asMA + 2*(nu_1_asMA)^(3);
mu4_asMA = nu_4_asMA - 4* nu_3_asMA * nu_1_asMA + ...
           6*nu_2_asMA*(nu_1_asMA)^(2) - 3*(nu_1_asMA)^(4);

%% skewness and kurtosis
skew_asMA = mu3_asMA/sd_asMA^(3);
kurt_asMA = mu4_asMA/sd_asMA^(4);

%% ACVF at lag 1
gamma_0_asMA = var_asMA;                   % ACVF(0)
gamma_1_asMA = 0.5*var_w*(beta_p+beta_m);  % ACVF(1) 

%  0.5*(var_w/pi)*(beta_p+beta_m); % lag-1 autocovariance
rho_asMA = gamma_1_asMA/gamma_0_asMA; %  ACF(1)

disp('Parameters: beta_p, beta_m')
disp(par)

moments = [nu_1_asMA, nu_2_asMA, nu_3_asMA, nu_4_asMA];
disp('Non-central moments asMA11: nu_1  nu_2  nu_3  nu_4')
disp(moments)

var_skew_kurt = [var_asMA, skew_asMA, kurt_asMA, rho_asMA];
disp('asMA11: var  skew  kurt  rho(ACF(1)) ')
disp(var_skew_kurt)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% PART 2  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  SPECIAL CASE stationary SETAR(2;1,1) process %%%%%%%%%%
%%    Y_{t} =  alpha Y_{t-1}  + varepsilon_{t} if Y_{t-1} <= 0
%%    Y_{t} = -alpha Y_{t-1}  + varepsilon_{t} if Y_{t-1} > 0
%%  with varepsilon_{t} ~ N(0,1)
%%  alpha = beta/sqrt(1+beta^(2)); or 1-alpha^2 = 1/(1+beta^2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% clear all; clc

Umax   = 201;
beta_p = 0.5;
%% alpha  = beta_p/sqrt(1+beta_p^(2));  % relationship between asMA(1,1)
                                        % and SETAR(2;1,1)
alpha  = 0.5;

par    = [alpha, beta_p];

alpha2 = alpha*alpha;
alpha3 = sqrt(2*(1-alpha2)/pi);

sigma     = 1;
sigma2    = sigma*sigma;
pdf_SETAR = zeros(1,Umax);

for i=1:Umax 
  u = -10+(i-1)*0.1;
  f_setar = sigma*alpha3 * exp( -(1-alpha2)*u*u / (2*sigma2) ) * normcdf(-alpha*u/sigma2); 
  pdf_SETAR(i) =  f_setar; % stationary density SETAR(2;1,1)
  index(i)     =  u;
end

figure(2)
plot(index,pdf_SETAR)
hold on 
plot(index,pdf_asMA2)
hold off

diff = pdf_SETAR' - pdf_asMA2';
norm(diff)  % ans = 0.0671 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  In the SPECIAL CASE, moments of asMA are the same as moments of
%%%%  SETAR211 with changes in sign for odd moments nu_1 and nu_3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
beta_p = 0.5;
alpha  = 0.5;
par    = [alpha, beta_p];

alpha2 = alpha*alpha;
alpha3 = sqrt(2*(1-alpha2)/pi);

mu         = sqrt(2/pi)*alpha/sqrt(1-alpha2);   % mean, 1st central moment, sign +
mu2        = 1/(1-alpha2);                      % second central moment
var_SETAR  = mu2 - mu*mu;                       % variance 
var2       = (pi-2*alpha2)/(pi*(1-alpha2));     % verification of var(u)
v2         = (1/(1-alpha2))*(1-(2*alpha2)/pi);
sd         = sqrt(v2);

ku  = (1-alpha2)/alpha2;
J_0 = sqrt(pi/(2*ku));
J_1 = -1/( ku*sqrt(1+ku) );
J_2 = J_0/ku;
J_3 = ( (2/ku)+(1/(ku+1)) )*J_1;
J_4 = (3/ku)*J_2;

% Non-central moments: Thm 4.2 (Andel, 1984)
% nu_{p} = alpha^{-p-1}[2(1-alpha^2)/pi]^{1/2} * J_{p}

nu_1_SETAR = -(alpha)^(-2) * sqrt( 2* (1-alpha2)/pi )*J_1; % sign -
nu_2_SETAR =  (alpha)^(-3) * sqrt( 2* (1-alpha2)/pi )*J_2;
nu_3_SETAR = -(alpha)^(-4) * sqrt( 2* (1-alpha2)/pi )*J_3; % sign -
nu_4_SETAR =  (alpha)^(-5) * sqrt( 2* (1-alpha2)/pi )*J_4;

%% Central moments
mu3_SETAR = nu_3_SETAR - 3* nu_2_SETAR * nu_1_SETAR + 2*(nu_1_SETAR)^(3);
mu4_SETAR = nu_4_SETAR - 4* nu_3_SETAR * nu_1_SETAR + ...
           6*nu_2_SETAR*(nu_1_SETAR)^(2) - 3*(nu_1_SETAR)^(4);

%% skewness and kurtosis
skew_SETAR = mu3_SETAR/sd^(3);
kurt_SETAR = mu4_SETAR/sd^(4);

%% ACF at lag 1
rho = ( alpha*pi + 2*alpha2*sqrt(1-alpha2) ...
       - 2*alpha2 - 2*alpha*atan(sqrt(alpha2^(-1)-1)) )/(pi-2*alpha2);

%% ACVF at lag 1 = E[XY] : Andel, et al. (1984, Thm 4.3)
%% gamma_1 = 0.4060

gamma_1 = ((1-alpha2)^(-1)) * alpha +  ...
           2*alpha2/(pi*(1-alpha2)^(1/2))  - ...
           2*alpha*atan((alpha2^(-1)-1)^(0.5))/(pi*(1-alpha2));

EX = -(alpha)*sqrt(2/pi)/sqrt(1-alpha2);  % Mean mu -0.460658865961781

%%%%  ACVF at lag 1 
acvf_1 = (gamma_1 - EX^(2));           % acvf_1 = 0.1938
rho2   = (gamma_1 - EX^(2))/var_SETAR; % rho = rho2 = 0.1729

disp('Parameters for SETAR(2;1,1) = asMA(1,1): alpha, beta_p')
disp(par)

moments_SETAR = [nu_1_SETAR, nu_2_SETAR, nu_3_SETAR, nu_4_SETAR];
disp('SETAR211: nu_1  nu_2  nu_3  nu_4')
disp(moments_SETAR)

var_skew_kurt_rho = [var_SETAR, skew_SETAR, kurt_SETAR, rho, rho2, gamma_1];
disp('SETAR211: var  skew  kurt  rho at lag 1  rho2   gamma_1')
disp(var_skew_kurt_rho)

%%%%%%%%%%%%%%%%%%%%  END MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
